@@ -7,7 +7,7 @@ use diagnostics;
 use mro 'c3';
 use English qw(-no_match_vars);
 use Carp;
-our $VERSION = 6.2;
+our $VERSION = 6.3;
 use Fatal qw( close );
 use Array::Contains;
 #---AUTOPRAGMAEND---
@@ -50,6 +50,7 @@ sub new {
 
     $self->{needreconnect} = 1;
     $self->{inlines} = [];
+    $self->{firstconnect} = 1;
 
     $self->{memcached_compatibility} = 0;
 
@@ -72,6 +73,17 @@ sub reconnect {
         delete $self->{socket};
     }
 
+    if($self->{firstconnect}) {
+        $self->{firstconnect} = 0;
+    } else {
+        # Not our first connection (=real reconnect).
+        # wait a short random time before reconnecting. In case all
+        # clients got disconnected, we want to avoid having all clients reconnect
+        # at the exact same time
+        my $waittime = rand(4000)/1000;
+        sleep($waittime);
+    }
+
     my $socket = IO::Socket::IP->new(
         PeerHost => $self->{server},
         PeerPort => $self->{port},
@@ -92,6 +104,8 @@ sub reconnect {
     $self->{outbuffer} = '';
     $self->{serverinfo} = 'UNKNOWN';
     $self->{needreconnect} = 0;
+    $self->{firstline} = 1;
+    $self->{headertimeout} = time + 15;
 
     # Do *not* nuke "inlines" array, since it may hold "QUIT" messages that the client wants to handle, for example, to re-issue
     # "LISTEN" commands.
@@ -100,12 +114,8 @@ sub reconnect {
     # Startup "handshake". As everything else, this is asyncronous, both server and
     # client send their respective version strings and then wait to recieve their counterparts
     # Also, this part is REQUIRED, just to make sure we actually speek to CLACKS protocol
-    #
-    # In this implementation, we wait until we recieve the server header. If we don't recieve it within
-    # 20 seconds, we time out and fail.
     $self->{outbuffer} .= 'CLACKS ' . $self->{clientname} . "\r\n";
     $self->{outbuffer} .= 'OVERHEAD A ' . $self->{authtoken} . "\r\n";
-    my $timeout = time + 20;
 
     return;
 }
@@ -199,17 +209,35 @@ sub doNetwork {
         if($char eq "\r") {
             next;
         } elsif($char eq "\n") {
-            if($self->{inbuffer} ne 'NOP') { # Just drop "No OPerations" packets, only used by server to
+            if($self->{inbuffer} eq 'NOP') { # Just drop "No OPerations" packets, only used by server to
                                              # verify that the connection is still active
-                #print STDERR "GOT ", $self->{inbuffer}, "\n";
-                push @{$self->{inlines}}, $self->{inbuffer};
+                #$self->{firstline}
+                $self->{inbuffer} = '';
+                next;
             }
+
+            if($self->{firstline}) {
+                if($self->{inbuffer} !~ /^CLACKS\ /) {
+                    # Whoops, not a clacks server or something gone wrong with the protocol
+                    $self->{needreconnect} = 1;
+                    return 0;
+                } else {
+                    $self->{firstline} = 0;
+                }
+            }
+
+            push @{$self->{inlines}}, $self->{inbuffer};
             $self->{inbuffer} = '';
         } else {
             $self->{inbuffer} .= $char;
         }
     }
 
+    if($self->{firstline} && $self->{headertimeout} < time) {
+        $self->{needreconnect} = 1;
+        return 0;
+    }
+
     return $workCount;
 }
 
