@@ -7,7 +7,7 @@ use diagnostics;
 use mro 'c3';
 use English qw(-no_match_vars);
 use Carp;
-our $VERSION = 6.2;
+our $VERSION = 6.3;
 use Fatal qw( close );
 use Array::Contains;
 #---AUTOPRAGMAEND---
@@ -115,6 +115,10 @@ sub init {
         $self->{config}->{interclacksreconnecttimeout} = 30;
     }
 
+    if(!defined($self->{config}->{authtimeout})) {
+        $self->{config}->{authtimeout} = 15;
+    }
+
     my @tcpsockets;
 
     foreach my $ip (@{$config->{ip}}) {
@@ -256,6 +260,7 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
                         interclacksclient => 1,
                         lastinterclacksping => time,
                         lastmessage => time,
+                        authtimeout => time + $self->{config}->{authtimeout},
                         authok => 0,
                     );
                     $clients{$mcid} = \%tmp;
@@ -320,6 +325,7 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
                     interclacksclient => 0,
                     lastinterclacksping => 0,
                     lastmessage => time,
+                    authtimeout => time + $self->{config}->{authtimeout},
                     authok => 0,
                 );
                 if(0 && $self->{isDebugging}) {
@@ -346,11 +352,13 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
                 if($clients{$cid}->{lastping} > 0 && $clients{$cid}->{lastping} < $pingtime) {
                     syswrite($clients{$cid}->{socket}, "\r\nTIMEOUT\r\n");
                     push @toremove, $cid;
+                    next;
                 }
             } else {
                 if($clients{$cid}->{lastping} < $interclackspingtime) {
                     syswrite($clients{$cid}->{socket}, "\r\nTIMEOUT\r\n");
                     push @toremove, $cid;
+                    next;
                 }
             }
 
@@ -358,6 +366,11 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
                 $clients{$cid}->{lastinterclacksping} = time;
                 $clients{$cid}->{outbuffer} .= "PING\r\n";
             }
+
+            if(!$clients{$cid}->{authok} && $clients{$cid}->{authtimeout} < time) {
+                # Authentication timeout!
+                push @toremove, $cid;
+            }
         }
 
         # ...and disconnect them
@@ -385,7 +398,7 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
 
         if(!(scalar keys %clients)) {
             # No clients to handle, let's sleep and try again later
-            sleep(0.2);
+            sleep(0.1);
             next;
         }
 
@@ -473,6 +486,7 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
                             } else {
                                 $clients{$cid}->{authok} = 0;
                                 $clients{$cid}->{outbuffer} .= "OVERHEAD F Login failed!\r\n";
+                                push @toremove, $cid; # Disconnect the client
                             }
                         }
 
